@@ -5,6 +5,7 @@
 package source
 
 import (
+	"context"
 	"fmt"
 	"os"
 	"regexp"
@@ -53,30 +54,14 @@ import (
 	errors "golang.org/x/xerrors"
 )
 
-const (
-	// CommandGenerate is a gopls command to run `go test` for a specific test function.
-	CommandTest = "test"
-
-	// CommandGenerate is a gopls command to run `go generate` for a directory.
-	CommandGenerate = "generate"
-
-	// CommandTidy is a gopls command to run `go mod tidy` for a module.
-	CommandTidy = "tidy"
-
-	// CommandVendor is a gopls command to run `go mod vendor` for a module.
-	CommandVendor = "vendor"
-
-	// CommandUpgradeDependency is a gopls command to upgrade a dependency.
-	CommandUpgradeDependency = "upgrade_dependency"
-
-	// CommandRegenerateCfgo is a gopls command to regenerate cgo definitions.
-	CommandRegenerateCgo = "regenerate_cgo"
-)
-
 // DefaultOptions is the options that are used for Gopls execution independent
 // of any externally provided configuration (LSP initialization, command
 // invokation, etc.).
 func DefaultOptions() Options {
+	var commands []string
+	for _, c := range Commands {
+		commands = append(commands, c.Name)
+	}
 	return Options{
 		ClientOptions: ClientOptions{
 			InsertTextFormat:                  protocol.PlainTextTextFormat,
@@ -94,33 +79,29 @@ func DefaultOptions() Options {
 					protocol.SourceOrganizeImports: true,
 					protocol.QuickFix:              true,
 					protocol.RefactorRewrite:       true,
+					protocol.RefactorExtract:       true,
 				},
 				Mod: {
 					protocol.SourceOrganizeImports: true,
 				},
 				Sum: {},
 			},
-			SupportedCommands: []string{
-				CommandTest,
-				CommandTidy,
-				CommandUpgradeDependency,
-				CommandGenerate,
-				CommandRegenerateCgo,
-			},
+			SupportedCommands: commands,
 		},
 		UserOptions: UserOptions{
 			Env:                     os.Environ(),
 			HoverKind:               FullDocumentation,
 			LinkTarget:              "pkg.go.dev",
+			LinksInHover:            true,
 			Matcher:                 Fuzzy,
 			SymbolMatcher:           SymbolFuzzy,
 			DeepCompletion:          true,
 			UnimportedCompletion:    true,
 			CompletionDocumentation: true,
 			EnabledCodeLens: map[string]bool{
-				CommandGenerate:          true,
-				CommandUpgradeDependency: true,
-				CommandRegenerateCgo:     true,
+				CommandGenerate.Name:          true,
+				CommandUpgradeDependency.Name: true,
+				CommandRegenerateCgo.Name:     true,
 			},
 		},
 		DebuggingOptions: DebuggingOptions{
@@ -132,7 +113,7 @@ func DefaultOptions() Options {
 		},
 		Hooks: Hooks{
 			ComputeEdits:         myers.ComputeEdits,
-			URLRegexp:            regexp.MustCompile(`(http|ftp|https)://([\w_-]+(?:(?:\.[\w_-]+)+))([\w.,@?^=%&:/~+#-]*[\w@?^=%&/~+#-])?`),
+			URLRegexp:            urlRegexp(),
 			DefaultAnalyzers:     defaultAnalyzers(),
 			TypeErrorAnalyzers:   typeErrorAnalyzers(),
 			ConvenienceAnalyzers: convenienceAnalyzers(),
@@ -203,9 +184,17 @@ type UserOptions struct {
 	// StaticCheck enables additional analyses from staticcheck.io.
 	StaticCheck bool
 
-	// LinkTarget is the website used for documentation.
+	// LinkTarget is the website used for documentation. If empty, no link is
+	// provided.
 	LinkTarget string
 
+	// LinksInHover toggles the presence of links to documentation in hover.
+	LinksInHover bool
+
+	// ImportShortcut specifies whether import statements should link to
+	// documentation or go to definitions. The default is both.
+	ImportShortcut ImportShortcut
+
 	// LocalPrefix is used to specify goimports's -local behavior.
 	LocalPrefix string
 
@@ -215,6 +204,10 @@ type UserOptions struct {
 	// SymbolMatcher specifies the type of matcher to use for symbol requests.
 	SymbolMatcher SymbolMatcher
 
+	// SymbolStyle specifies what style of symbols to return in symbol requests
+	// (package qualified, fully qualified, etc).
+	SymbolStyle SymbolStyle
+
 	// DeepCompletion allows completion to perform nested searches through
 	// possible candidates.
 	DeepCompletion bool
@@ -229,6 +222,25 @@ type UserOptions struct {
 	// Placeholders adds placeholders to parameters and structs in completion
 	// results.
 	Placeholders bool
+
+	// Gofumpt indicates if we should run gofumpt formatting.
+	Gofumpt bool
+}
+
+type ImportShortcut int
+
+const (
+	Both ImportShortcut = iota
+	Link
+	Definition
+)
+
+func (s ImportShortcut) ShowLinks() bool {
+	return s == Both || s == Link
+}
+
+func (s ImportShortcut) ShowDefinition() bool {
+	return s == Both || s == Definition
 }
 
 type completionOptions struct {
@@ -251,6 +263,7 @@ type Hooks struct {
 	DefaultAnalyzers     map[string]Analyzer
 	TypeErrorAnalyzers   map[string]Analyzer
 	ConvenienceAnalyzers map[string]Analyzer
+	GofumptFormat        func(ctx context.Context, src []byte) ([]byte, error)
 }
 
 func (o Options) AddDefaultAnalyzer(a *analysis.Analyzer) {
@@ -303,6 +316,14 @@ const (
 	SymbolCaseSensitive
 )
 
+type SymbolStyle int
+
+const (
+	PackageQualifiedSymbols = SymbolStyle(iota)
+	FullyQualifiedSymbols
+	DynamicSymbols
+)
+
 type HoverKind int
 
 const (
@@ -449,6 +470,20 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 			o.SymbolMatcher = SymbolCaseInsensitive
 		}
 
+	case "symbolStyle":
+		style, ok := result.asString()
+		if !ok {
+			break
+		}
+		switch style {
+		case "full":
+			o.SymbolStyle = FullyQualifiedSymbols
+		case "dynamic":
+			o.SymbolStyle = DynamicSymbols
+		default:
+			o.SymbolStyle = PackageQualifiedSymbols
+		}
+
 	case "hoverKind":
 		hoverKind, ok := result.asString()
 		if !ok {
@@ -472,6 +507,21 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 	case "linkTarget":
 		result.setString(&o.LinkTarget)
 
+	case "linksInHover":
+		result.setBool(&o.LinksInHover)
+
+	case "importShortcut":
+		var s string
+		result.setString(&s)
+		switch s {
+		case "both":
+			o.ImportShortcut = Both
+		case "link":
+			o.ImportShortcut = Link
+		case "definition":
+			o.ImportShortcut = Definition
+		}
+
 	case "analyses":
 		result.setBoolMap(&o.UserEnabledAnalyses)
 
@@ -502,6 +552,9 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 	case "tempModfile":
 		result.setBool(&o.TempModfile)
 
+	case "gofumpt":
+		result.setBool(&o.Gofumpt)
+
 	// Replaced settings.
 	case "experimentalDisabledAnalyses":
 		result.State = OptionDeprecated
@@ -602,28 +655,49 @@ func (r *OptionResult) setString(s *string) {
 	}
 }
 
+// EnabledAnalyzers returns all of the analyzers enabled for the given
+// snapshot.
+func EnabledAnalyzers(snapshot Snapshot) (analyzers []Analyzer) {
+	for _, a := range snapshot.View().Options().DefaultAnalyzers {
+		if a.Enabled(snapshot) {
+			analyzers = append(analyzers, a)
+		}
+	}
+	for _, a := range snapshot.View().Options().TypeErrorAnalyzers {
+		if a.Enabled(snapshot) {
+			analyzers = append(analyzers, a)
+		}
+	}
+	for _, a := range snapshot.View().Options().ConvenienceAnalyzers {
+		if a.Enabled(snapshot) {
+			analyzers = append(analyzers, a)
+		}
+	}
+	return analyzers
+}
+
 func typeErrorAnalyzers() map[string]Analyzer {
 	return map[string]Analyzer{
 		fillreturns.Analyzer.Name: {
 			Analyzer:       fillreturns.Analyzer,
-			enabled:        true,
 			FixesError:     fillreturns.FixesError,
-			HighConfidence: false,
+			HighConfidence: true,
 		},
 		nonewvars.Analyzer.Name: {
 			Analyzer:   nonewvars.Analyzer,
-			enabled:    true,
 			FixesError: nonewvars.FixesError,
+			enabled:    true,
 		},
 		noresultvalues.Analyzer.Name: {
 			Analyzer:   noresultvalues.Analyzer,
-			enabled:    true,
 			FixesError: noresultvalues.FixesError,
+			enabled:    true,
 		},
 		undeclaredname.Analyzer.Name: {
 			Analyzer:   undeclaredname.Analyzer,
-			enabled:    true,
 			FixesError: undeclaredname.FixesError,
+			Command:    CommandUndeclaredName,
+			enabled:    true,
 		},
 	}
 }
@@ -632,7 +706,8 @@ func convenienceAnalyzers() map[string]Analyzer {
 	return map[string]Analyzer{
 		fillstruct.Analyzer.Name: {
 			Analyzer: fillstruct.Analyzer,
-			enabled:  false,
+			Command:  CommandFillStruct,
+			enabled:  true,
 		},
 	}
 }
@@ -676,3 +751,10 @@ func defaultAnalyzers() map[string]Analyzer {
 		simplifyslice.Analyzer.Name:        {Analyzer: simplifyslice.Analyzer, enabled: true, HighConfidence: true},
 	}
 }
+
+func urlRegexp() *regexp.Regexp {
+	// Ensure links are matched as full words, not anywhere.
+	re := regexp.MustCompile(`\b(http|ftp|https)://([\w_-]+(?:(?:\.[\w_-]+)+))([\w.,@?^=%&:/~+#-]*[\w@?^=%&/~+#-])?\b`)
+	re.Longest()
+	return re
+}
